package uulm.teamname.marvelous.gamelibrary.entities;

import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import uulm.teamname.marvelous.gamelibrary.IntVector2;
import uulm.teamname.marvelous.gamelibrary.json.ingame.deserialize.EntityDeserializer;
import uulm.teamname.marvelous.gamelibrary.json.ingame.serialize.EntitySerializer;

import java.util.Objects;

/** Represents an abstract entity. */
@JsonDeserialize(using = EntityDeserializer.class)
@JsonSerialize(using = EntitySerializer.class)
public abstract class Entity {
    /** Whether or not the entity blocks movement */
    protected boolean solid = false;

    /** Whether or not the entity blocks vision */
    protected boolean opaque = false;

    /** The position of the entity */
    protected IntVector2 position;

    /** The {@link EntityID} of the entity */
    public final EntityID id;

    /**
     * Constructs a new {@link Entity}.
     * @param id The {@link EntityID} of the entity
     * @param position The position of the entity
     */
    protected Entity(EntityID id, IntVector2 position) {
        this.position = position;
        this.id = id;
    }

    /**
     * Clones the {@link Entity}.
     * @return A new entity cloned from this one
     */
    public abstract Entity clone();

    public boolean blocksMovement() {
        return solid;
    }

    public boolean blocksVision() {
        return opaque;
    }

    public IntVector2 getPosition() {
        return position;
    }

    public void setPosition(IntVector2 position) {
        this.position = position;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        Entity entity = (Entity) o;
        return Objects.equals(position, entity.position) && Objects.equals(id, entity.id);
    }

    @Override
    public int hashCode() {
        return Objects.hash(position, id);
    }
}

