package uulm.teamname.marvelous.gamelibrary.entities;

import uulm.teamname.marvelous.gamelibrary.IntVector2;

import java.util.ArrayList;

/** Represents an NPC inside the game. */
public class NPC extends Entity {
    /** The {@link StatType#MP} of the NPC */
    public final Stat mp;

    /** The {@link Inventory} of the NPC */
    public final Inventory inventory;

    /**
     * Constructs a new {@link NPC}.
     * @param id The {@link EntityID} of the NPC
     * @param position The position of the NPC
     * @param inventory The starting inventory the NPC should have
     */
    public NPC(EntityID id, IntVector2 position, int maxMP, ArrayList<StoneType> inventory) {
        super(id, position);
        solid = true;
        opaque = true;
        this.inventory = new Inventory(inventory);
        this.mp = new Stat(StatType.MP, maxMP);
    }

    /**
     * Constructs a new {@link NPC} with an empty inventory.
     * @param id The {@link EntityID} of the NPC
     * @param position The position of the NPC
     * @param maxMP The maximum MP of the NPC
     */
    public NPC(EntityID id, IntVector2 position, int maxMP) {
        this(id, position, maxMP, new ArrayList<>());
    }

    /**
     * Constructs a new {@link NPC} with an empty inventory.
     * @param id The {@link EntityID} of the NPC
     * @param position The position of the NPC
     * @param inventory The starting inventory of the NPC
     */
    public NPC(EntityID id, IntVector2 position, ArrayList<StoneType> inventory) {
        this(id, position, 0, inventory);
    }

    /**
     * Constructs a new {@link NPC} with an empty inventory.
     * @param id The {@link EntityID} of the NPC
     * @param position The position of the NPC
     */
    public NPC(EntityID id, IntVector2 position) {
        this(id, position, 0);
    }

    @Override
    public NPC clone() {
        NPC clone = new NPC(id, position, mp.getMax());
        clone.mp.setValue(mp.getValue());
        for(StoneType stone: inventory) {
            clone.inventory.addStone(stone);
        }
        return clone;
    }

    @Override
    public String toString() {
        return "NPC{" +
                "mp=" + mp +
                ", inventory=" + inventory +
                ", position=" + position +
                '}';
    }
}
