package uulm.teamname.marvelous.gamelibrary.entities;

import uulm.teamname.marvelous.gamelibrary.IntVector2;

import java.util.Objects;

/** Represents a rock entity on the map. */
public class Rock extends Entity {
    /** The maximum hp of the rock */
    public final int maxHP;

    /** The current hp of the rock */
    private int hp;

    /**
     * Constructs a new {@link Rock}.
     * @param id The {@link EntityID} of the rock
     * @param position The position of the rock
     * @param hp The hp of the rock
     */
    public Rock(EntityID id, IntVector2 position, int hp) {
        super(id, position);
        solid = true;
        opaque = true;
        this.maxHP = hp;
        this.hp = hp;
    }

    public int getHp() {
        return hp;
    }

    public void decreaseHp(int damage) {
        hp -= damage;
    }

    @Override
    public Rock clone() {
        Rock clone = new Rock(id, position, maxHP);
        clone.hp = hp;
        return clone;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        if (!super.equals(o)) return false;
        Rock rock = (Rock) o;
        return maxHP == rock.maxHP && hp == rock.hp;
    }



    @Override
    public int hashCode() {
        return Objects.hash(super.hashCode(), maxHP, hp);
    }


    @Override
    public String toString() {
        return "Rock{" +
                "maxHP=" + maxHP +
                ", hp=" + hp +
                ", position=" + position +
                '}';
    }
}
