package uulm.teamname.marvelous.gamelibrary.events;

import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import uulm.teamname.marvelous.gamelibrary.IntVector2;
import uulm.teamname.marvelous.gamelibrary.entities.EntityID;
import uulm.teamname.marvelous.gamelibrary.entities.StoneType;
import uulm.teamname.marvelous.gamelibrary.json.ingame.deserialize.EventDeserializer;
import uulm.teamname.marvelous.gamelibrary.json.ingame.serialize.EventSerializer;

import java.util.Objects;

/**
 * Represents a character event for: {@link EventType#MeleeAttackEvent}, {@link EventType#RangedAttackEvent}, {@link EventType#MoveEvent}, {@link EventType#ExchangeInfinityStoneEvent}, {@link EventType#UseInfinityStoneEvent}.
 */
@JsonDeserialize(using = EventDeserializer.class)
@JsonSerialize(using = EventSerializer.class)
public class CharacterEvent extends Event {
    public EntityID originEntity = null;
    public EntityID targetEntity = null;
    public IntVector2 originField = null;
    public IntVector2 targetField = null;
    public Integer amount = null;
    public StoneType stoneType = null;

    @Override
    public boolean check() {
        if(!super.check()) {
            return false;
        }

        switch(type) {
            // Melee- and ranged attacks need the same properties (except for the type)
            case MeleeAttackEvent:
            case RangedAttackEvent:
                if (this.originField == null ||
                        this.targetField == null ||
                        this.originEntity == null ||
                        this.targetEntity == null) {
                    return false;
                }
                break;

            // Exchanging and using InfinityStones both uses the same keys. Hereby, using a stone like the
            // RealityStone causes the stone to be used on oneself
            case ExchangeInfinityStoneEvent:
            case UseInfinityStoneEvent:
                if (this.originField == null ||
                        this.targetField == null ||
                        this.originEntity == null ||
                        this.targetEntity == null ||
                        this.stoneType == null) {
                    return false;
                }
                break;

            // MoveEvents take an originField, a targetField, and an originEntity.
            case MoveEvent:
                if (this.originField == null ||
                        this.targetField == null ||
                        this.originEntity == null) {
                    return false;
                }
                break;
        }

        return true;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        if (!super.equals(o)) return false;
        CharacterEvent that = (CharacterEvent) o;
        return Objects.equals(originEntity, that.originEntity) && Objects.equals(targetEntity, that.targetEntity) && Objects.equals(originField, that.originField) && Objects.equals(targetField, that.targetField) && Objects.equals(amount, that.amount) && Objects.equals(stoneType, that.stoneType);
    }

    @Override
    public int hashCode() {
        return Objects.hash(super.hashCode(), originEntity, targetEntity, originField, targetField, amount, stoneType);
    }

    @Override
    public String toString() {
        return "CharacterEvent{" +
                "eventType=" + this.type +
                ", originEntity=" + originEntity +
                ", targetEntity=" + targetEntity +
                ", originField=" + originField +
                ", targetField=" + targetField +
                ", amount=" + amount +
                ", stoneType=" + stoneType +
                '}';
    }
}
