package uulm.teamname.marvelous.gamelibrary.events;

import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import uulm.teamname.marvelous.gamelibrary.json.ingame.deserialize.EventDeserializer;
import uulm.teamname.marvelous.gamelibrary.json.ingame.serialize.EventSerializer;

import java.util.HashMap;
import java.util.Objects;

/** Represents a custom event ({@link EventType#CustomEvent}). */
@JsonDeserialize(using = EventDeserializer.class)
@JsonSerialize(using = EventSerializer.class)
public class CustomEvent extends Event {
    public String teamIdentifier;
    public HashMap<String, Object> customContent;

    @Override
    public boolean check() {
        if(!super.check()) {
            return false;
        }

        switch(type) {
            // CustomEvent only requires the custom data.
            case CustomEvent:
                if (this.customContent == null) {
                    return false;
                }
                break;
        }

        return true;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        if (!super.equals(o)) return false;
        CustomEvent that = (CustomEvent) o;
        return Objects.equals(teamIdentifier, that.teamIdentifier) && Objects.equals(customContent, that.customContent);
    }

    @Override
    public int hashCode() {
        return Objects.hash(super.hashCode(), teamIdentifier, customContent);
    }

    @Override
    public String toString() {
        return "CustomEvent{" +
                "eventType=" + this.type +
                ", teamIdentifier='" + teamIdentifier + '\'' +
                ", customContent=" + customContent +
                '}';
    }
}
