package uulm.teamname.marvelous.gamelibrary.events;

import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import uulm.teamname.marvelous.gamelibrary.messages.server.EventMessage;
import uulm.teamname.marvelous.gamelibrary.json.ingame.deserialize.EventDeserializer;
import uulm.teamname.marvelous.gamelibrary.json.ingame.serialize.EventSerializer;

import java.util.Objects;

/** Represents an abstract event sent inside a {@link EventMessage} between client and server. */
@JsonDeserialize(using = EventDeserializer.class)
@JsonSerialize(using = EventSerializer.class)
public abstract class Event {
    public EventType type;

    public Event type(EventType type) {
        this.type = type;
        return this;
    }

    /**
     * Checks whether the event contains all necessary properties according to its {@link EventType}.
     * @return True if the event has all required properties set, otherwise false
     */
    public boolean check() {
        return type != null;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        Event event = (Event) o;
        return type == event.type;
    }

    @Override
    public int hashCode() {
        return Objects.hash(type);
    }
}
