package uulm.teamname.marvelous.gamelibrary.gamelogic;

import uulm.teamname.marvelous.gamelibrary.IntVector2;
import uulm.teamname.marvelous.gamelibrary.entities.EntityID;
import uulm.teamname.marvelous.gamelibrary.entities.EntityType;
import uulm.teamname.marvelous.gamelibrary.entities.StoneType;

import java.util.ArrayList;

/** Represents a game state view containing getters for all the properties of a {@link GameState}. */
public class GameStateView {
    /** The managed {@link GameState} */
    private final GameState state;

    /**
     * Constructs a new {@link GameStateView}.
     * @param state A reference to the {@link GameState} to be viewable
     */
    public GameStateView(GameState state) {
        this.state = state;
    }

    public IntVector2 getMapSize() {
        return state.mapSize;
    }

    public EntityManager getEntities() {
        return state.entities;
    }

    public int getRoundNumber() {
        return state.roundNumber;
    }

    public int getTurnNumber() {
        return state.turnNumber;
    }

    public EntityID getActiveCharacter() {
        return state.activeCharacter;
    }

    public ArrayList<EntityID> getTurnOrder() {
        return state.turnOrder;
    }

    public boolean isWon() {
        return state.won;
    }

    public int getStoneCooldown(StoneType stone) {
        return state.stoneCooldown.getCooldown(stone);
    }

    public EntityType getCurrentOvertimeWinner() {
        return state.winConditions.getWinner();
    }
}
