package uulm.teamname.marvelous.gamelibrary.json.ingame.deserialize;

import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.ObjectCodec;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.JsonDeserializer;
import com.fasterxml.jackson.databind.JsonNode;
import uulm.teamname.marvelous.gamelibrary.IntVector2;
import uulm.teamname.marvelous.gamelibrary.entities.Entity;
import uulm.teamname.marvelous.gamelibrary.entities.EntityID;
import uulm.teamname.marvelous.gamelibrary.entities.StoneType;
import uulm.teamname.marvelous.gamelibrary.events.Event;
import uulm.teamname.marvelous.gamelibrary.events.EventBuilder;
import uulm.teamname.marvelous.gamelibrary.events.EventType;

import static uulm.teamname.marvelous.gamelibrary.json.JsonNodeUnwrapper.unwrap;

import java.io.IOException;
import java.util.HashMap;

public class EventDeserializer extends JsonDeserializer<Event> {

    @Override
    public Event deserialize(JsonParser p, DeserializationContext ctxt) throws IOException {
        ObjectCodec codec = p.getCodec();
        JsonNode node = codec.readTree(p);

        JsonNode currentNode = null;

        EventBuilder builder;
        EventType eventType;

        if (!node.has("eventType")) {
            throw new IOException("Event had wrong format: no EventType found");
        } else {
            eventType = EventType.valueOf(node.get("eventType").asText());
            builder = new EventBuilder(eventType);
        }

//            eventType = EventType.valueOf(node.get("eventType").asText());
//
//            switch (eventType) {
//                case Ack:
//                case Nack:
//                case GamestateEvent:
//                    result = new GamestateEvent();
//                    result.type = eventType;
//
//                    if ((currentNode = node.get("activeCharacter")) != null) {
//                        ((GamestateEvent) result).activeCharacter = mapper
//                                .readValue(currentNode.toString(), EntityID.class);
//                    }
//
//                    if ((currentNode = node.get("turnOrder")) != null) {
//                        ((GamestateEvent) result).turnOrder = mapper
//                                .readValue(currentNode.toString(), EntityID[].class);
//                    }
//
//
////                    currentNode = node.get("entities");
////                    ((GameStateEvent) result).entities = currentNode == null
////                            ? null
////                            : mapper.readValue(currentNode.asText(), Entity[].class);
////
////                    currentNode = node.get("turnOrder");
////                    ((GameStateEvent) result).turnOrder = currentNode == null
////                            ? null
////                            : mapper.readValue(currentNode.asText(), EntityID[].class);
//            }

        // necessary because of exact type mismatch
        var stoneTypeID = unwrap(node.get("stoneType"), EntityID.class, codec);

        builder.withTargetEntity(unwrap(node.get("targetEntity"), EntityID.class, codec))
                .withTargetField(unwrap(node.get("targetField"), IntVector2.class, codec))
                .withAmount(unwrap(node.get("amount"), Integer.class, codec))
                .withEntity(unwrap(node.get("entity"), Entity.class, codec))

                .withOriginEntity(unwrap(node.get("originEntity"), EntityID.class, codec))
                .withOriginField(unwrap(node.get("originField"), IntVector2.class, codec))
                .withStoneType(stoneTypeID != null ? StoneType.valueOf(stoneTypeID.id) : null)

                .withTeleportedEntity(unwrap(node.get("teleportedEntity"), EntityID.class, codec))
                .withOriginPortal(unwrap(node.get("originPortal"), EntityID.class, codec))
                .withTargetPortal(unwrap(node.get("targetPortal"), EntityID.class, codec))

                .withRoundCount(unwrap(node.get("roundCount"), Integer.class, codec))
                .withTurnCount(unwrap(node.get("turnCount"), Integer.class, codec))
                .withCharacterOrder(unwrap(node.get("characterOrder"), EntityID[].class, codec))
                .withNextCharacter(unwrap(node.get("nextCharacter"), EntityID.class, codec))

                .withPlayerWon(unwrap(node.get("playerWon"), Integer.class, codec))

                .withMessage(unwrap(node.get("message"), String.class, codec))
                .withTimeLeft(unwrap(node.get("timeLeft"), Integer.class, codec))

                .withEntities(unwrap(node.get("entities"), Entity[].class, codec))
                .withTurnOrder(unwrap(node.get("turnOrder"), EntityID[].class, codec))
                .withMapSize(unwrap(node.get("mapSize"), IntVector2.class, codec))
                .withActiveCharacter(unwrap(node.get("activeCharacter"), EntityID.class, codec))
                .withStoneCooldowns(unwrap(node.get("stoneCooldowns"), Integer[].class, codec))
                .withWinCondition(unwrap(node.get("winCondition"), Boolean.class, codec))

                .withTeamIdentifier(unwrap(node.get("teamIdentifier"), String.class, codec))
                .withCustomContent(unwrap(node.get("customContent"), HashMap.class, codec));


        builder.withTargetEntity(unwrap(node.get("targetEntity"), EntityID.class, codec));
        builder.withTargetField(unwrap(node.get("targetField"), IntVector2.class, codec));

        switch (eventType) {
            case Ack, Nack, GamestateEvent -> { return builder.buildGameStateEvent(); }
            case CustomEvent -> { return builder.buildCustomEvent(); }
            case DestroyedEntityEvent,
                    TakenDamageEvent,
                    ConsumedAPEvent,
                    ConsumedMPEvent,
                    SpawnEntityEvent,
                    HealedEvent -> { return builder.buildEntityEvent(); }
            case MeleeAttackEvent,
                    RangedAttackEvent,
                    MoveEvent,
                    UseInfinityStoneEvent,
                    ExchangeInfinityStoneEvent -> { return builder.buildCharacterEvent(); }
            case TeleportedEvent -> { return builder.buildTeleportedEvent(); }
            case RoundSetupEvent,
                    TurnEvent,
                    WinEvent,
                    TurnTimeoutEvent,
                    TimeoutWarningEvent,
                    TimeoutEvent,
                    PauseStartEvent,
                    PauseStopEvent,
                    DisconnectEvent -> { return builder.buildGameEvent(); }
        }
        return null;
    }


}
