package uulm.teamname.marvelous.gamelibrary.json.ingame.deserialize;

import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.ObjectCodec;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.JsonDeserializer;
import com.fasterxml.jackson.databind.JsonNode;
import uulm.teamname.marvelous.gamelibrary.IntVector2;
import uulm.teamname.marvelous.gamelibrary.entities.EntityID;
import uulm.teamname.marvelous.gamelibrary.entities.StoneType;
import uulm.teamname.marvelous.gamelibrary.requests.*;

import java.io.IOException;
import java.util.HashMap;
import java.util.Optional;

import static uulm.teamname.marvelous.gamelibrary.json.JsonNodeUnwrapper.unwrap;

public class RequestDeserializer extends JsonDeserializer<Request> {

    @Override
    public Request deserialize(JsonParser p, DeserializationContext ctxt) throws IOException {
        ObjectCodec codec = p.getCodec();
        JsonNode node = codec.readTree(p);

        JsonNode requestTypeNode;
        RequestType requestType;
        if ((requestTypeNode = node.get("requestType")) != null) {
            requestType = RequestType.valueOf(requestTypeNode.textValue());
        } else {
            throw new IOException("No RequestType found. JSON does not represent a request");
        }

        var requestBuilder = new RequestBuilder(requestType)
                .withValue(unwrap(node.get("value"), int.class, codec))
                .withOriginEntity(unwrap(node.get("originEntity"), EntityID.class, codec))
                .withTargetEntity(unwrap(node.get("targetEntity"), EntityID.class, codec))
                .withOriginField(unwrap(node.get("originField"), IntVector2.class, codec))
                .withTargetField(unwrap(node.get("targetField"), IntVector2.class, codec))
                .withStoneType(Optional.ofNullable(unwrap(node.get("stoneType"), EntityID.class, codec))
                        .map(x -> StoneType.valueOf(x.id))
                        .orElse(null))
                .withTeamIdentifier(unwrap(node.get("teamIdentifier"), String.class, codec))
                .withCustomContent(unwrap(node.get("teamIdentifier"), HashMap.class, codec));

        switch (requestType) {
            case DisconnectRequest,
                    Req,
                    PauseStartRequest,
                    PauseStopRequest,
                    EndRoundRequest -> {
                return requestBuilder.buildGameRequest();
            }
            case ExchangeInfinityStoneRequest,
                    MeleeAttackRequest,
                    MoveRequest,
                    RangedAttackRequest,
                    UseInfinityStoneRequest -> {
                return requestBuilder.buildCharacterRequest();
            }
        }

        return null; // if there is no request type
    }
}
