package uulm.teamname.marvelous.gamelibrary.messages.client;

import com.fasterxml.jackson.annotation.JsonIgnore;
import uulm.teamname.marvelous.gamelibrary.events.Event;
import uulm.teamname.marvelous.gamelibrary.messages.BasicMessage;
import uulm.teamname.marvelous.gamelibrary.messages.MessageType;
import uulm.teamname.marvelous.gamelibrary.requests.Request;

import jakarta.validation.constraints.NotEmpty;
import jakarta.validation.constraints.NotNull;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Objects;

/** Represents a message sent between client and server and contains all possible data. */
public class RequestMessage extends BasicMessage {

    /** The list of {@link Event Events} sent inside the message. */
    @NotNull(message = "No Requests found")
    @NotEmpty(message = "No Requests found")
    public Request[] messages;

    /** The type of the custom content sent with the message. */
    public String customContentType;

    /** The decoded contents of the custom content sent. */
    public HashMap<String, Object> customContent;

    public RequestMessage() {
        messageType = MessageType.REQUESTS;
    }

    @Override @JsonIgnore
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        RequestMessage that = (RequestMessage) o;
        return messageType == that.messageType && Arrays.equals(messages, that.messages) && Objects.equals(customContentType, that.customContentType) && Objects.equals(customContent, that.customContent);
    }

    @Override @JsonIgnore
    public int hashCode() {
        int result = Objects.hash(messageType, customContentType, customContent);
        result = 31 * result + Arrays.hashCode(messages);
        return result;
    }

    @Override @JsonIgnore
    public String toString() {
        return "MessageStructure{" +
                "messageType=" + messageType +
                ", messages=" + Arrays.toString(messages) +
                ", customContentType='" + customContentType + '\'' +
                ", customContent=" + customContent +
                '}';
    }
}
