package uulm.teamname.marvelous.gamelibrary.messages.server;

import uulm.teamname.marvelous.gamelibrary.config.CharacterProperties;
import uulm.teamname.marvelous.gamelibrary.messages.BasicMessage;
import uulm.teamname.marvelous.gamelibrary.messages.MessageType;

import jakarta.validation.constraints.NotEmpty;
import jakarta.validation.constraints.NotNull;
import jakarta.validation.constraints.Size;
import java.util.Arrays;
import java.util.Objects;

// @JsonDeserialize(using = GameAssignmentDeserializer.class)
public class GameAssignmentMessage extends BasicMessage {

    /** The ID of the game that the client is connected to. What this is used for is kind of unknown. */
    @NotEmpty
    public String gameID;

    /** The characters the player can choose from */
    @NotNull
    @Size(min = 12, max = 12, message = "Character Selection doesn't have 12 booleans")
    public CharacterProperties[] characterSelection;

    public GameAssignmentMessage() {
        messageType = MessageType.GAME_ASSIGNMENT;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        if (!super.equals(o)) return false;
        GameAssignmentMessage that = (GameAssignmentMessage) o;
        return Objects.equals(gameID, that.gameID) && Arrays.equals(characterSelection, that.characterSelection);
    }

    @Override
    public int hashCode() {
        int result = Objects.hash(super.hashCode(), gameID);
        result = 31 * result + Arrays.hashCode(characterSelection);
        return result;
    }

    @Override
    public String toString() {
        return "GameAssignmentMessage{" +
                "messageType=" + messageType +
                ", optionals='" + optionals + '\'' +
                ", gameID='" + gameID + '\'' +
                ", characterSelection=" + Arrays.toString(characterSelection) +
                '}';
    }
}
