package uulm.teamname.marvelous.gamelibrary.json.config;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import net.jqwik.api.*;
import net.jqwik.api.lifecycle.BeforeProperty;
import org.junit.jupiter.api.Test;
import uulm.teamname.marvelous.gamelibrary.config.CharacterConfig;
import uulm.teamname.marvelous.gamelibrary.config.CharacterProperties;
import uulm.teamname.marvelous.gamelibrary.json.JSON;

import java.util.Arrays;
import java.util.HashSet;
import java.util.concurrent.ThreadLocalRandom;

import static org.assertj.core.api.Assertions.*;

class CharacterConfigJSONTest {
    private static final String json = """
            {
              "characters": [
                {
                  "characterID": 1,
                  "name": "Rocket Raccoon",
                  "HP": 100,
                  "MP": 2,
                  "AP": 2,
                  "meleeDamage": 10,
                  "rangeCombatDamage": 30,
                  "rangeCombatReach": 5
                },
                {
                  "characterID": 2,
                  "name": "Quicksilver",
                  "HP": 100,
                  "MP": 6,
                  "AP": 1,
                  "meleeDamage": 10,
                  "rangeCombatDamage": 10,
                  "rangeCombatReach": 3
                },
                {
                  "characterID": 3,
                  "name": "Hulk",
                  "HP": 100,
                  "MP": 2,
                  "AP": 2,
                  "meleeDamage": 10,
                  "rangeCombatDamage": 30,
                  "rangeCombatReach": 5
                },
                {
                  "characterID": 4,
                  "name": "Black Widow",
                  "HP": 100,
                  "MP": 6,
                  "AP": 1,
                  "meleeDamage": 10,
                  "rangeCombatDamage": 10,
                  "rangeCombatReach": 3
                },
                {
                  "characterID": 5,
                  "name": "Hawkeye",
                  "HP": 100,
                  "MP": 6,
                  "AP": 1,
                  "meleeDamage": 10,
                  "rangeCombatDamage": 10,
                  "rangeCombatReach": 3
                },
                {
                  "characterID": 6,
                  "name": "Captain America",
                  "HP": 100,
                  "MP": 2,
                  "AP": 2,
                  "meleeDamage": 10,
                  "rangeCombatDamage": 30,
                  "rangeCombatReach": 5
                },
                {
                  "characterID": 7,
                  "name": "Spiderman",
                  "HP": 100,
                  "MP": 6,
                  "AP": 1,
                  "meleeDamage": 10,
                  "rangeCombatDamage": 10,
                  "rangeCombatReach": 3
                },
                {
                  "characterID": 8,
                  "name": "Dr. Strange",
                  "HP": 100,
                  "MP": 2,
                  "AP": 2,
                  "meleeDamage": 10,
                  "rangeCombatDamage": 30,
                  "rangeCombatReach": 5
                },
                {
                  "characterID": 9,
                  "name": "Iron Man",
                  "HP": 100,
                  "MP": 6,
                  "AP": 1,
                  "meleeDamage": 10,
                  "rangeCombatDamage": 10,
                  "rangeCombatReach": 3
                },
                {
                  "characterID": 10,
                  "name": "Black Panther",
                  "HP": 100,
                  "MP": 6,
                  "AP": 1,
                  "meleeDamage": 10,
                  "rangeCombatDamage": 10,
                  "rangeCombatReach": 3
                },
                {
                  "characterID": 11,
                  "name": "Thor",
                  "HP": 100,
                  "MP": 2,
                  "AP": 2,
                  "meleeDamage": 10,
                  "rangeCombatDamage": 30,
                  "rangeCombatReach": 5
                },
                {
                  "characterID": 12,
                  "name": "Captain Marvel",
                  "HP": 100,
                  "MP": 6,
                  "AP": 1,
                  "meleeDamage": 10,
                  "rangeCombatDamage": 10,
                  "rangeCombatReach": 3
                },
                {
                  "characterID": 13,
                  "name": "Groot",
                  "HP": 100,
                  "MP": 2,
                  "AP": 2,
                  "meleeDamage": 10,
                  "rangeCombatDamage": 30,
                  "rangeCombatReach": 5
                },
                {
                  "characterID": 14,
                  "name": "Starlord",
                  "HP": 100,
                  "MP": 6,
                  "AP": 1,
                  "meleeDamage": 10,
                  "rangeCombatDamage": 10,
                  "rangeCombatReach": 3
                },
                {
                  "characterID": 15,
                  "name": "Gamora",
                  "HP": 100,
                  "MP": 6,
                  "AP": 1,
                  "meleeDamage": 10,
                  "rangeCombatDamage": 10,
                  "rangeCombatReach": 3
                },
                {
                  "characterID": 16,
                  "name": "Ant Man",
                  "HP": 100,
                  "MP": 2,
                  "AP": 2,
                  "meleeDamage": 10,
                  "rangeCombatDamage": 30,
                  "rangeCombatReach": 5
                },
                {
                  "characterID": 17,
                  "name": "Vision",
                  "HP": 100,
                  "MP": 6,
                  "AP": 1,
                  "meleeDamage": 10,
                  "rangeCombatDamage": 10,
                  "rangeCombatReach": 3
                },
                {
                  "characterID": 18,
                  "name": "Deadpool",
                  "HP": 100,
                  "MP": 2,
                  "AP": 2,
                  "meleeDamage": 10,
                  "rangeCombatDamage": 30,
                  "rangeCombatReach": 5
                },
                {
                  "characterID": 19,
                  "name": "Loki",
                  "HP": 100,
                  "MP": 6,
                  "AP": 1,
                  "meleeDamage": 10,
                  "rangeCombatDamage": 10,
                  "rangeCombatReach": 3
                },
                {
                  "characterID": 20,
                  "name": "Silver Surfer",
                  "HP": 100,
                  "MP": 6,
                  "AP": 1,
                  "meleeDamage": 10,
                  "rangeCombatDamage": 10,
                  "rangeCombatReach": 3
                },
                {
                  "characterID": 21,
                  "name": "Mantis",
                  "HP": 100,
                  "MP": 2,
                  "AP": 2,
                  "meleeDamage": 10,
                  "rangeCombatDamage": 30,
                  "rangeCombatReach": 5
                },
                {
                  "characterID": 22,
                  "name": "Ghost Rider",
                  "HP": 100,
                  "MP": 6,
                  "AP": 1,
                  "meleeDamage": 10,
                  "rangeCombatDamage": 10,
                  "rangeCombatReach": 3
                },
                {
                  "characterID": 23,
                  "name": "Jesica Jones",
                  "HP": 100,
                  "MP": 2,
                  "AP": 2,
                  "meleeDamage": 10,
                  "rangeCombatDamage": 30,
                  "rangeCombatReach": 5
                },
                {
                  "characterID": 24,
                  "name": "Scarlet Witch",
                  "HP": 100,
                  "MP": 6,
                  "AP": 1,
                  "meleeDamage": 10,
                  "rangeCombatDamage": 10,
                  "rangeCombatReach": 3
                }
              ]
            }
            """;

    @Test
    void disjointSubsetGenerationWorks() {
        var characterConfig = JSON.parseCharacterConfig(json);
        System.out.println(characterConfig.get().characters.length);
        var tuple = characterConfig.get().getDisjointSetsOfPropertiesOfSize(6);
        assertThat(tuple.item1).doesNotHaveDuplicates().hasSize(6);
        assertThat(tuple.item2).doesNotHaveDuplicates().hasSize(6);
        assertThat(tuple.item1)
                .doesNotContainAnyElementsOf(Arrays.asList(tuple.item2));
    }

    ObjectMapper mapper;

    @BeforeProperty
    void beforeProperty() {
        mapper = new ObjectMapper();
    }

    @Property
    void characterConfigsAreDeserializedProperly(@ForAll("CharacterConfigs") CharacterConfig config)
            throws JsonProcessingException {
        var jsonRepresentingCharacterConfig = getJsonOfCharacterConfig(config);
        assertThat(mapper.readValue(jsonRepresentingCharacterConfig, CharacterConfig.class))
                .isEqualTo(config);
    }


    @Property
    void characterConfigsAreSerializedProperly(@ForAll("CharacterConfigs") CharacterConfig config)
            throws JsonProcessingException {
        var jsonRepresentingCharacterConfig = getJsonOfCharacterConfig(config);
        assertThat(mapper.writeValueAsString(config))
                .isEqualTo(jsonRepresentingCharacterConfig);
    }

    @Provide("CharacterConfigs")
    Arbitrary<CharacterConfig> characterConfigs() {
        HashSet<Integer> usedIDs = new HashSet<>();
        HashSet<String> usedNames = new HashSet<>();

        return characterProperties()
                .filter(property -> usedIDs.add(property.characterID))
                .filter(property -> usedNames.add(property.name))
                .collect(list -> list.size() >= ThreadLocalRandom.current().nextInt(1999) + 1)
                .map(list -> {
                    var config = new CharacterConfig();
                    config.characters = list.toArray(new CharacterProperties[0]);
                    return config;
                });
    }


    @Provide("CharacterProperties")
    Arbitrary<CharacterProperties> characterProperties() {
        return Combinators.combine(
                Arbitraries.integers()
                        .greaterOrEqual(0)
                        .tuple5(),
                Arbitraries.integers()
                        .greaterOrEqual(0)
                        .tuple2(),
                Arbitraries.strings()
                        .alpha()
                        .withChars(' ')
                        .ofMinLength(3)
                        .ofMaxLength(3))
                .as((fiveInts, twoInts, name) -> {
                    var properties = new CharacterProperties();
                    properties.characterID = fiveInts.get1();
                    properties.name = name;
                    properties.HP = fiveInts.get2();
                    properties.MP = fiveInts.get3();
                    properties.AP = fiveInts.get4();
                    properties.meleeDamage = fiveInts.get5();
                    properties.rangedDamage = twoInts.get1();
                    properties.attackRange = twoInts.get2();
                    return properties;
                });
    }


    private String getJsonOfProperties(
            Integer id,
            String name,
            Integer HP,
            Integer MP,
            Integer AP,
            Integer meleeDamage,
            Integer rangeCombatDamage,
            Integer rangeCombatReach
    ) {
        return String.format("""
                {
                  "characterID": %d,
                  "name": "%s",
                  "HP": %d,
                  "MP": %d,
                  "AP": %d,
                  "meleeDamage": %d,
                  "rangeCombatDamage": %d,
                  "rangeCombatReach": %d
                }
                """.replace(" ", ""),
                id,
                name,
                HP,
                MP,
                AP,
                meleeDamage,
                rangeCombatDamage,
                rangeCombatReach).replace("\n", "");
    }

    String getJsonOfCharacterConfig(CharacterConfig config) {
        var jsonRepresentingCharacterConfig = new StringBuilder("""
                {
                "characters":[
                """.replace("\n", ""));

        for (CharacterProperties prop: config.characters) {
            jsonRepresentingCharacterConfig.append(getJsonOfProperties(
                    prop.characterID,
                    prop.name,
                    prop.HP,
                    prop.MP,
                    prop.AP,
                    prop.meleeDamage,
                    prop.rangedDamage,
                    prop.attackRange
            )).append(',');
        }

        jsonRepresentingCharacterConfig.setLength(jsonRepresentingCharacterConfig.length() - 1);
        return jsonRepresentingCharacterConfig.append(']').append('}').toString();


    }
}
