package uulm.teamname.marvelous.gamelibrary.json.config;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import net.jqwik.api.*;
import net.jqwik.api.constraints.*;
import net.jqwik.api.lifecycle.BeforeProperty;
import uulm.teamname.marvelous.gamelibrary.config.CharacterProperties;

import static org.assertj.core.api.Assertions.*;

class CharacterPropertiesJSONTest {

    ObjectMapper mapper;

    @BeforeProperty
    void beforeProperty() {
        mapper = new ObjectMapper();
    }

    @Property
    void characterPropertiesGetDeserializedProperly(
            @ForAll("Positive Integers") Integer id,
            @ForAll @NotBlank @StringLength(min = 3, max = 50) @AlphaChars @Chars({' '}) String name,
            @ForAll ("Positive Integers") Integer HP,
            @ForAll ("Positive Integers") Integer MP,
            @ForAll ("Positive Integers") Integer AP,
            @ForAll ("Positive Integers") Integer meleeDamage,
            @ForAll ("Positive Integers") Integer rangeCombatDamage,
            @ForAll ("Positive Integers") Integer rangeCombatReach) throws JsonProcessingException {

        var values = getPropertiesAndJson(
                id,
                name,
                HP,
                MP,
                AP,
                meleeDamage,
                rangeCombatDamage,
                rangeCombatReach);


        assertThat(mapper.readValue(values.get2(), CharacterProperties.class))
                .isEqualTo(values.get1());
    }


    @Property
    void characterPropertiesGetSerializedProperly(
            @ForAll("Positive Integers") Integer id,
            @ForAll @NotBlank @StringLength(min = 3, max = 50) @AlphaChars @Chars({' '}) String name,
            @ForAll ("Positive Integers") Integer HP,
            @ForAll ("Positive Integers") Integer MP,
            @ForAll ("Positive Integers") Integer AP,
            @ForAll ("Positive Integers") Integer meleeDamage,
            @ForAll ("Positive Integers") Integer rangeCombatDamage,
            @ForAll ("Positive Integers") Integer rangeCombatReach) throws JsonProcessingException {

        var values = getPropertiesAndJson(
                id,
                name,
                HP,
                MP,
                AP,
                meleeDamage,
                rangeCombatDamage,
                rangeCombatReach);


        assertThat(mapper.writeValueAsString(values.get1()))
                .isEqualTo(values.get2());
    }

    @Provide("Positive Integers")
    Arbitrary<Integer> positiveIntegers() {
        return Arbitraries.integers().greaterOrEqual(0);
    }

    private Tuple.Tuple2<CharacterProperties, String> getPropertiesAndJson(
            Integer id,
            String name,
            Integer HP,
            Integer MP,
            Integer AP,
            Integer meleeDamage,
            Integer rangeCombatDamage,
            Integer rangeCombatReach
    ) {
        var properties = new CharacterProperties();
        properties.characterID = id;
        properties.name = name;
        properties.HP = HP;
        properties.MP = MP;
        properties.AP = AP;
        properties.meleeDamage = meleeDamage;
        properties.rangedDamage = rangeCombatDamage;
        properties.attackRange = rangeCombatReach;

        var jsonRepresentingProperties = String.format("""
                {
                  "characterID": %d,
                  "name": "%s",
                  "HP": %d,
                  "MP": %d,
                  "AP": %d,
                  "meleeDamage": %d,
                  "rangeCombatDamage": %d,
                  "rangeCombatReach": %d
                }
                """.replace(" ", ""),
                id,
                name,
                HP,
                MP,
                AP,
                meleeDamage,
                rangeCombatDamage,
                rangeCombatReach).replace("\n", "");

        return Tuple.of(properties, jsonRepresentingProperties);
    }

}
