package uulm.teamname.marvelous.gamelibrary.json.ingame.serialize;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
//import net.jqwik.api.*;
//import net.jqwik.api.lifecycle.BeforeProperty;
import net.jqwik.api.*;
import net.jqwik.api.lifecycle.BeforeProperty;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import uulm.teamname.marvelous.gamelibrary.IntVector2;
import uulm.teamname.marvelous.gamelibrary.entities.*;
import uulm.teamname.marvelous.gamelibrary.events.*;

import java.util.HashMap;

import static org.assertj.core.api.Assertions.*;

public class EventSerializerTest {

    EntityID[] turns;
    EntityID turn;
    Entity[] entities;
    Entity entity;
    EventBuilder filled;


    @Test
    void serializeGamestateEvent() throws JsonProcessingException {

        var mapper = new ObjectMapper();

        GamestateEvent gse = new GamestateEvent();
        gse.type = EventType.Ack;

        var jsonRepresentingGSE = """
                {
                "eventType":"Ack"
                }""".replace("\n", "");

        assertThat(mapper.writeValueAsString(gse))
              .isEqualTo(jsonRepresentingGSE);

        gse.type = EventType.Nack;

         jsonRepresentingGSE = """
                {
                "eventType":"Nack"
                }""".replace("\n", "");

        assertThat(mapper.writeValueAsString(gse))
                .isEqualTo(jsonRepresentingGSE);


        gse.type = EventType.GamestateEvent;

        EventBuilder eb = new EventBuilder(EventType.GamestateEvent);

        GamestateEvent gse2 = eb // no exception if all properties present
                .withEntities(entities)
                .withTurnOrder(turns)
                .withMapSize(new IntVector2(42, 24))
                .withActiveCharacter(turn)
                .withStoneCooldowns(new Integer[] {6, 5, 4, 3, 2, 1})
                .withWinCondition(false)
                .buildGameStateEvent();

       // gse2.entities = [new EntityID(EntityType.NPC, 1),new IntVector2(1,1)]
        Entity[] e = {new NPC(new EntityID(EntityType.NPC, 1), new IntVector2(1,1))};
        gse2.entities = e;
        EntityID[] eid = {new EntityID(EntityType.NPC, 1)};
        gse2.turnOrder= eid;
        var ac=  new EntityID(EntityType.P1, 2);
        gse2.activeCharacter = ac;
        System.out.println(mapper.writeValueAsString(gse2));

        jsonRepresentingGSE = """
                {"eventType":"GamestateEvent",
                "entities":[{"entityType":"NPC","ID":1,"MP":0,"stones":[],
                "position":[1,1]}],"mapSize":[42,24],
                "turnOrder":[{"entityID":"NPC","ID":1}],
                "activeCharacter":{"entityID":"P1","ID":2},
                "stoneCooldowns":[6,5,4,3,2,1],
                "winCondition":false
                }""".replace("\n", "");

        assertThat(mapper.writeValueAsString(gse2))
                .isEqualTo(jsonRepresentingGSE);

    }

    @Test
    void serializeEntityEvent() throws JsonProcessingException {

        var mapper = new ObjectMapper();

        EntityEvent ee = new EntityEvent();
        ee.type = EventType.SpawnEntityEvent;
        ee.entity = new NPC(new EntityID(EntityType.NPC, 1), new IntVector2(1,1));

        var jsonRepresentingEE = """
                {
                "eventType":"SpawnEntityEvent",
                "entity":{"entityType":"NPC","ID":1,"MP":0,"stones":[],"position":[1,1]}
                }""".replace("\n", "");

     //   System.out.println(mapper.writeValueAsString(ee));
        assertThat(mapper.writeValueAsString(ee))
                .isEqualTo(jsonRepresentingEE);


        EntityEvent ee2 = new EntityEvent();
        ee2.type = EventType.TakenDamageEvent;
        ee2.targetEntity = new EntityID(EntityType.P1,5);
        ee2.targetField =  new IntVector2(1,1);
        ee2.amount = 10;

       // System.out.println(mapper.writeValueAsString(ee2));

        jsonRepresentingEE = """
                {
                "eventType":"TakenDamageEvent",
                "targetEntity":{"entityID":"P1","ID":5},
                "targetField":[1,1],
                "amount":10
                }""".replace("\n", "");

       // System.out.println(mapper.writeValueAsString(ee2));
        assertThat(mapper.writeValueAsString(ee2))
                .isEqualTo(jsonRepresentingEE);

        EntityEvent ee3 = new EntityEvent();
        ee3.type = EventType.DestroyedEntityEvent;
        ee3.targetEntity = new EntityID(EntityType.P1,5);
        ee3.targetField =  new IntVector2(1,1);


        jsonRepresentingEE = """
                {
                "eventType":"DestroyedEntityEvent",
                "targetEntity":{"entityID":"P1","ID":5},
                "targetField":[1,1]
                }""".replace("\n", "");

        //System.out.println(mapper.writeValueAsString(ee3));
        assertThat(mapper.writeValueAsString(ee3))
                .isEqualTo(jsonRepresentingEE);
    }

    @Test
    void serializeCharacterEvent() throws JsonProcessingException {

        var mapper = new ObjectMapper();

        CharacterEvent ce = new CharacterEvent();
        ce.type = EventType.MeleeAttackEvent;
        ce.originEntity =  new EntityID(EntityType.P2,7);
        ce.targetEntity = new EntityID(EntityType.P1,5);
        ce.originField =  new IntVector2(1,2);
        ce.targetField =  new IntVector2(2,2);

        var jsonRepresentingEE = """
                {
                "eventType":"MeleeAttackEvent",
                "originEntity":{"entityID":"P2","ID":7},
                "targetEntity":{"entityID":"P1","ID":5},
                "originField":[1,2],"targetField":[2,2]
                }""".replace("\n", "");

      //  System.out.println(mapper.writeValueAsString(ce));
       assertThat(mapper.writeValueAsString(ce))
              .isEqualTo(jsonRepresentingEE);

        CharacterEvent ce2 = new CharacterEvent();
        ce2.type = EventType.UseInfinityStoneEvent;
        ce2.originEntity =  new EntityID(EntityType.P2,7);
        ce2.targetEntity = new EntityID(EntityType.P1,5);
        ce2.originField =  new IntVector2(1,1);
        ce2.targetField =  new IntVector2(2,2);
        ce2.stoneType = StoneType.MindStone;

        jsonRepresentingEE = """
                {
                "eventType":"UseInfinityStoneEvent",
                "originEntity":{"entityID":"P2","ID":7},
                "targetEntity":{"entityID":"P1","ID":5},
                "originField":[1,1],
                "targetField":[2,2],
                "stoneType":{"entityID":"InfinityStones","ID":1}
                }""".replace("\n", "");

        System.out.println(mapper.writeValueAsString(ce2));

        assertThat(mapper.writeValueAsString(ce2))
                .isEqualTo(jsonRepresentingEE);

    }

    @Test
    void serializeGameEvent() throws JsonProcessingException {

        var mapper = new ObjectMapper();

        GameEvent ge = new GameEvent();
        ge.type = EventType.RoundSetupEvent;
        ge.roundCount =5;
        EntityID[] eid = {new EntityID(EntityType.NPC, 1)};
        ge.characterOrder = eid;

        var jsonRepresentingEE = """
               {
               "eventType":"RoundSetupEvent",
               "roundCount":5,
               "characterOrder":[{"entityID":"NPC","ID":1}]
               }""".replace("\n", "");

       // System.out.println(mapper.writeValueAsString(ge));

        assertThat(mapper.writeValueAsString(ge))
            .isEqualTo(jsonRepresentingEE);

        GameEvent ge2 = new GameEvent();
        ge2.type = EventType.TurnEvent;
        ge2.turnCount =5;
        ge2.nextCharacter = new EntityID(EntityType.NPC, 1);

         jsonRepresentingEE = """
                {
                "eventType":"TurnEvent",
                "turnCount":5,
                "nextCharacter":{"entityID":"NPC","ID":1}
                }""".replace("\n", "");

       // System.out.println(mapper.writeValueAsString(ge2));
          assertThat(mapper.writeValueAsString(ge2))
               .isEqualTo(jsonRepresentingEE);

        GameEvent ge3 = new GameEvent();
        ge3.type = EventType.WinEvent;
        ge3.playerWon =1;

        jsonRepresentingEE = """
                {
                "eventType":"WinEvent",
                "playerWon":1
                }""".replace("\n", "");

       //  System.out.println(mapper.writeValueAsString(ge3));
        assertThat(mapper.writeValueAsString(ge3))
                .isEqualTo(jsonRepresentingEE);

        GameEvent ge4 = new GameEvent();
        ge4.type = EventType.TimeoutWarningEvent;
        ge4.message = "warning";
        ge4.timeLeft =  1;

        jsonRepresentingEE = """
                {
                "eventType":"TimeoutWarningEvent",
                "message":"warning",
                "timeLeft":1
                }""".replace("\n", "");

          //System.out.println(mapper.writeValueAsString(ge4));
        assertThat(mapper.writeValueAsString(ge4))
                .isEqualTo(jsonRepresentingEE);

        GameEvent ge5 = new GameEvent();
        ge5.type = EventType.TimeoutEvent;
        ge5.message = "timeout";


        jsonRepresentingEE = """
                {
                "eventType":"TimeoutEvent",
                "message":"timeout"
                }""".replace("\n", "");

       // System.out.println(mapper.writeValueAsString(ge5));
        assertThat(mapper.writeValueAsString(ge5))
                .isEqualTo(jsonRepresentingEE);
    }

    @Test
    void serializeCustomEvent() throws JsonProcessingException {
        var mapper = new ObjectMapper();

        CustomEvent ce = new CustomEvent();
        ce.type = EventType.CustomEvent;
        ce.teamIdentifier ="identity";
        ce.customContent = new HashMap<>(1, 1);


        var jsonRepresentingEE = """
                {
                "eventType":"CustomEvent",
                "teamIdentifier":"identity",
                "customContent":{}
                }""".replace("\n", "");

        // System.out.println(mapper.writeValueAsString(ce));
          assertThat(mapper.writeValueAsString(ce))
               .isEqualTo(jsonRepresentingEE);
    }


    ObjectMapper mapper;

    @BeforeProperty
    void beforeProperty() {
        mapper = new ObjectMapper();
    }


    @Property
    void gameEventsGetDeserializedProperly() {
        var jsonRepresentingEvent = "";
    }


    @Provide("filledEventBuilders")
    private Arbitrary<EventBuilder> filledEventBuilders() {
    return null;
    }




    @Provide("entityIDs")
    private Arbitrary<EntityID> entityIDs() {
        var entityTypes = Arbitraries.of(EntityType.class);
        return Combinators.combine(entityTypes, Arbitraries.integers().greaterOrEqual(0))
                .as((type, id) -> {
                    int actualID;
                    if (type == EntityType.P1 || type == EntityType.P2 || type == EntityType.InfinityStones) {
                        actualID = id % 6;
                    } else if (type == EntityType.NPC) {
                        actualID = id % 3;
                    } else {
                        actualID = id;
                    }
                    return new EntityID(type, id);
                });
    }

    @Provide("randomPositions")
    private Arbitrary<IntVector2> randomPositions() {
        return Arbitraries.integers()
                .greaterOrEqual(0)
                .tuple2()
                .map((pos) -> new IntVector2(pos.get1(), pos.get2()));
    }

    @Provide("directions")
    private Arbitrary<IntVector2> directions() {
        return Arbitraries.integers()
                .between(-1, 1)
                .tuple2()
                .filter(pos -> pos.get1() != 0 && pos.get2() != 0) // eliminate zero vectors
                .map(pos -> new IntVector2(pos.get1(), pos.get2()));
    }


    @Provide("attackPositions")
    // Returns tuples of origin vectors (of an attack), and the vector pointing to the attack dir
    private Arbitrary<Tuple.Tuple2<IntVector2, IntVector2>> attackPositions() {
        return Combinators.combine(randomPositions(), directions().tuple5())
                .as((origin, dir) -> {
                    var attackField = origin.copy()
                            .add(dir.get1().scale(3))
                            .add(dir.get2().scale(3))
                            .add(dir.get3().scale(3))
                            .add(dir.get4().scale(3));
                    if (attackField.length() == 0) {
                        attackField.add(dir.get5().scale(3));
                    }
                    return Tuple.of(origin, attackField);
                });
    }

    @Example
    void teleportedEventSerializationTest() throws JsonProcessingException {

        var jsonRepresentingTeleportedEvent = """
                {
                "eventType":"TeleportedEvent",
                "teleportedEntity":{"entityID":"P1","ID":3},
                "originField":[6,2],
                "targetField":[11,14],
                "originPortal":{"entityID":"Portals","ID":0},
                "targetPortal":{"entityID":"Portals","ID":2}
                }
                """.replace("\n", "");

        var teleportedEvent = new EventBuilder(EventType.TeleportedEvent)
                .withTeleportedEntity(new EntityID(EntityType.P1, 3))
                .withOriginField(new IntVector2(6, 2))
                .withTargetField(new IntVector2(11, 14))
                .withOriginPortal(new EntityID(EntityType.Portals, 0))
                .withTargetPortal(new EntityID(EntityType.Portals, 2))
                .buildTeleportedEvent();

        assertThat(mapper.writeValueAsString(teleportedEvent))
                .isEqualTo(jsonRepresentingTeleportedEvent);
    }


}
